@@ -189,7 +189,7 @@ tk_pyobject_print(ClientData clientData, Tk_Window tkwin, char *widgRec,
 		  int offset, Tcl_FreeProc **freeProcPtr)
 {
     static char buf[100];
-    sprintf(buf, "%d", (int)*((PyObject**)(widgRec + offset)));
+    sprintf(buf, "%ld", (long)*((PyObject**)(widgRec + offset)));
     return buf;
 }
 
