@@ -75,9 +75,16 @@ PERFORMANCE OF THIS SOFTWARE.
 #include "Python.h"
 #include <ctype.h>
 
-/* Turn off thread support. Added for pax */
+/* Thread support hacking. Added for pax */
+#ifdef WITH_THREAD
+/* define to get readline/EventHook to work in a single-threaded but
+   thread-enabled program */
+#define PAX_THREAD_HACK 
+#endif
+/* turn off all other threading specific code here */
 #undef WITH_THREAD
 
+
 #ifdef WITH_THREAD
 #include "pythread.h"
 #endif
@@ -506,10 +513,6 @@ Tkapp_New(screenName, baseName, className, interactive)
 		return NULL;
 
 	v->interp = Tcl_CreateInterp();
-#if	TKMAJORMINOR == 8001
-	TclpInitLibraryPath(baseName);
-#	endif /* TKMAJORMINOR */
-
 
 #if defined(macintosh) && TKMAJORMINOR >= 8000
 	/* This seems to be needed since Tk 8.0 */
@@ -1914,7 +1917,9 @@ MyFileProc(clientData, mask)
 }
 #endif
 
-/*static PyThreadState *event_tstate = NULL;*/
+#if defined(WITH_THREAD) || defined(PAX_THREAD_HACK)
+static PyThreadState *event_tstate = NULL;
+#endif
 
 static int
 EventHook()
@@ -1922,7 +1927,7 @@ EventHook()
 #ifndef MS_WINDOWS
 	FHANDLE tfile;
 #endif
-#ifdef WITH_THREAD
+#if defined(WITH_THREAD) || defined(PAX_THREAD_HACK)
 	PyEval_RestoreThread(event_tstate);
 #endif
 	stdin_ready = 0;
@@ -1967,7 +1972,7 @@ EventHook()
 		excInCmd = valInCmd = trbInCmd = NULL;
 		PyErr_Print();
 	}
-#ifdef WITH_THREAD
+#if defined(WITH_THREAD) || defined(PAX_THREAD_HACK)
 	PyEval_SaveThread();
 #endif
 	return 0;
@@ -1980,7 +1985,7 @@ EnableEventHook()
 {
 #ifdef WAIT_FOR_STDIN
 	if (PyOS_InputHook == NULL) {
-#ifdef WITH_THREAD
+#if defined(WITH_THREAD) || defined(PAX_THREAD_HACK)
 		event_tstate = PyThreadState_Get();
 #endif
 		PyOS_InputHook = EventHook;
@@ -2061,6 +2066,10 @@ initpaxtkinter() /* function name changed for pax */
 	Tktt_Type.ob_type = &PyType_Type;
 	PyDict_SetItemString(d, "TkttType", (PyObject *)&Tktt_Type);
 
+	/* This helps the dynamic loader; in Unicode aware Tcl versions
+	   it also helps Tcl find its encodings. */
+	Tcl_FindExecutable(Py_GetProgramName());
+
 	if (PyErr_Occurred())
 		return;
 
