@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2000 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -19,7 +19,7 @@
 #type = Import
 #class_name = 'AILoader'
 #rx_magic = r'^%!PS-Adobe-[23]\.0'
-#tk_file_type = ("Adobe Illustrator", ('.ai', '.ps', '.eps'))
+#tk_file_type = ("Adobe Illustrator", '.ai')
 format_name = 'Adobe Illustrator'
 #unload = 1
 #standard_messages = 1
@@ -97,14 +97,14 @@ format_name = 'Adobe Illustrator'
 #   - Embedded EPS files
 
 
-''''_("Adobe Illustrator")' '''        
+(''"Adobe Illustrator")
 
 import string, re
 from math import pi, hypot
 from types import IntType, StringType, ListType
 import string
 
-import Image
+from PIL import Image
 
 import streamfilter
 
@@ -247,6 +247,8 @@ class AILoader(GenericLoader):
                  "&": 'ignore_operator',
                  "Bd": 'begin_gradient',
                  "Bs": 'gradient_stop',
+                 "BS": 'dummy_gradient_stop',
+                 "Br": 'gradient_ramps',
                  "BD": 'end_gradient',
                  "Bb": 'begin_gradient_instance',
                  "Bg": 'gradient_geometry',
@@ -306,6 +308,7 @@ class AILoader(GenericLoader):
         self.standard_encoding = encoding.adobe_standard
         self.font_map = {}
         self.guides = []
+        self.format_version = 0.0
 
     def __del__(self):
 	pass
@@ -534,6 +537,37 @@ class AILoader(GenericLoader):
         #color = apply(CreateRGBColor, color)
         self.stack.append((ramp_point / 100.0, color))
 
+    def dummy_gradient_stop(self, color_style, mid_point, ramp_point):
+        # same as gradient_stop but ignore all arguments. Illustrator 8
+        # seems to introduce this one for printing (i.e. Illustrator 8
+        # files with printing info contain the gradient stops *twice* in
+        # exactly the same format but once with the Bs operator and once
+        # with BS. I guess this has something to do with support for
+        # PostScript Level 3 and backwards compatibility with older
+        # Illustrator versions.
+        if color_style == 0:
+            # gray scale
+            k = self.pop()
+        elif color_style == 1:
+            # CMYK
+            self.pop_multi(4)
+        elif color_style == 2:
+            # RGB Color
+            self.pop_multi(7)
+        elif color_style == 3:
+            # CMYK Custom Color
+            self.pop_multi(6)
+        elif color_style == 4:
+            self.pop_multi(10)
+        else:
+            self.add_message(_("Unknown ColorStyle %d") % color_style)
+
+    def gradient_ramps(self, ramp_type):
+        # defines the ramp colors with a bunch of strings for printing.
+        # Here we just pop all the strings off the stack
+        num = (1, 4, 5, 6, 7, 8, 9)[ramp_type]
+        self.pop_multi(num)
+
     def end_gradient(self):
         self.make_array()
         array = self.pop()
@@ -547,6 +581,8 @@ class AILoader(GenericLoader):
                 array.reverse()
             name, type, ncolors = self.gradient_info
             self.gradients[name] = (type, array)
+        del self.stack[:]
+        #self.pop_to_mark()
 
     def begin_gradient_instance(self):
         self.in_gradient_instance = 1
@@ -755,8 +791,13 @@ class AILoader(GenericLoader):
     def begin_layer(self):
 	self.layer(_("Layer %d") % (len(self.composite_items) + 1))
 
-    def begin_ai_layer(self, visible, preview, enabled, printing, dimmed,
-                       has_mlm, color, red, green, blue):
+    def begin_ai_layer(self):
+        if self.format_version >= 4.0:
+            visible, preview, enabled, printing, dimmed, unused, has_mlm,\
+                 color, red, green, blue, unused, unused = self.pop_multi(13)
+        else:
+            visible, preview, enabled, printing, dimmed, has_mlm, \
+                     color, red, green, blue = self.pop_multi(10)
         color = CreateRGBColor(red / 255.0, green / 255.0, blue / 255.0)
         self.layer_kw_args = {'printable': printing,
                               'visible': visible,
@@ -968,6 +1009,8 @@ class AILoader(GenericLoader):
                     keyword = value
                 if keyword in ('EndProlog', 'BeginSetup'):
                     return keyword
+                if keyword[:14] == "AI5_FileFormat":
+                    self.format_version = string.atof(keyword[14:])
                 elif keyword == 'BeginProcSet':
                     # some ai files exported by corel draw don't have an
                     # EndProcSet comment after a BeginProcSet...
@@ -978,10 +1021,10 @@ class AILoader(GenericLoader):
                 return
 
     def Load(self):
-	funclist = self.get_compiled()
-	# binding frequently used functions to local variables speeds up
-	# the process considerably...
-	a = apply; t = tuple
+        funclist = self.get_compiled()
+        # binding frequently used functions to local variables speeds up
+        # the process considerably...
+        a = apply; t = tuple
         DSC = DSC_COMMENT; MAX = MAX_DATA_TOKEN; split = string.split
         stack = self.stack; push = self.stack.append
         unknown_operator = (None, None)
@@ -1009,6 +1052,8 @@ class AILoader(GenericLoader):
                     break
                 elif keyword == 'AI5_BeginPalette':
                     self.skip_to_dsc('AI5_EndPalette', 'EndSetup')
+                elif keyword == "AI8_BeginBrushPattern":
+                    self.skip_to_dsc('AI8_EndBrushPattern', 'EndSetup')
 
             elif token == END:
                 break
