@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2000 by Bernhard Herzog
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -229,9 +229,7 @@ class SVGHandler(saxlib.HandlerBase):
         self.basetrafo = self.trafo
 
     def parse_style(self, style):
-        #print 'parse_style'
-        parts = map(strip, split(style, ';'))
-        #print parts
+        parts = filter(None, map(strip, split(style, ';')))
         for part in parts:
             key, val = map(strip, split(part, ':', 1))
             self._print('style', key, val)
