@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1998, 1999 by Bernhard Herzog
+# Copyright (C) 1998, 1999, 2000 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -25,7 +25,7 @@ format_name = 'XFig'
 #standard_messages = 1
 ###End
 
-''''_("XFig")' '''
+(''"XFig")
 
 #
 # This is a simple import filter for Fig files.
@@ -401,11 +401,12 @@ class XFigLoader(SimplifiedLoader):
                 if not line:
                     raise SketchLoadError('Premature end of string')
                 string = string + line
+            globals = {'__builtins__': {}}
             try:
                 # using eval here might be a security hole!
-                string = eval('"""' + string[:-5] + '"""')
+                string = eval('"""' + string[:-5] + '"""', globals)
             except:
-                string = eval("'''" + string[:-5] + "'''")
+                string = eval("'''" + string[:-5] + "'''", globals)
         else:
             raise SketchLoadError('Invalid text string')
             
