@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1996, 1997, 1998, 1999 by Bernhard Herzog
+# Copyright (C) 1996, 1997, 1998, 1999, 2000 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -29,7 +29,7 @@ from types import TupleType
 import operator, string
 from math import pi
 
-import Image
+import PIL.Image
 
 import X
 from pax import PaxRegionType, IntersectMasks, CreateRegion
@@ -292,7 +292,7 @@ class SimpleGC(GCDevice):
 	self.gc.SetLineAttributes(round(width * self.scale), line, cap, join)
 	if dashes:
 	    if width < 1.0:
-		scale = 1.0
+		scale = self.scale
 	    else:
 		scale = width * self.scale
 	    dashes = map(operator.mul, dashes, [scale] * len(dashes))
@@ -537,9 +537,9 @@ class GraphicsDevice(SimpleGC, CommonDevice):
 	    cx, cy, cw, ch = self.clip_region.ClipBox()
 	    l = max(l, cx);	r = min(r, cx + cw)
 	    t = max(t, cy);	b = min(b, cy + ch)
-	if l > r or t > b:
+	if l >= r or t >= b:
 	    return None, None, None
-	image = Image.new('RGB', (r - l, b - t), (255, 255, 255))
+	image = PIL.Image.new('RGB', (r - l, b - t), (255, 255, 255))
 	trafo = Translation(-l, -t)(self.doc_to_win)
 	return image, trafo, (l, t)
 
@@ -565,8 +565,8 @@ class GraphicsDevice(SimpleGC, CommonDevice):
 	image, trafo, pos = self.get_pattern_image()
 	if image is None:
 	    return
-	x0, y0 = trafo.DocToWin(p0)
-	x1, y1 = trafo.DocToWin(p1)
+	x0, y0 = trafo(p0)
+	x1, y1 = trafo(p1)
 	#import time
 	#_t = time.clock()
 	_sketch.fill_axial_gradient(image.im, gradient.Colors(), x0,y0, x1,y1)
@@ -682,7 +682,7 @@ class GraphicsDevice(SimpleGC, CommonDevice):
 	    self.set_properties(properties)
 
     def IsOutlineActive(self):
-	return self.outline_mode
+	return not not self.outline_mode
 
 
     #
@@ -975,20 +975,27 @@ class GraphicsDevice(SimpleGC, CommonDevice):
 	self.complex_text = None
 
     def create_ximage(self):
+        global use_shm_images
 	if not self.ximage:
 	    w = self.widget
 	    if use_shm_images and not shm_images_supported:
 		warn(INTERNAL,
 		     'tried to use unsupported shared memory images\n')
-		global use_shm_images
 		use_shm_images = 0
 	    if use_shm_images:
                 try:
                     self.ximage = w.ShmCreateImage(w.depth, X.ZPixmap,
                                                    None, w.width, w.height, 1)
                 except:
+                    # Creating a shared memory image failed. Print a
+                    # message and don't use shmimages again. A likely
+                    # reason for this is that the test for shmimages in
+                    # pax succeeded but the ShmCreateImage here fails
+                    # because the limit for shm segments is too low, as
+                    # it is by default on Solaris.
                     warn(INTERNAL, _("Can't create shared memory image: %s"),
                          sys.exc_info()[1])
+                    use_shm_images = 0
             if not self.ximage:
 		self.ximage = self.create_sized_ximage(w.width, w.height)
 
