@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1996, 1997, 1998, 1999 by Bernhard Herzog
+# Copyright (C) 1996, 1997, 1998, 1999, 2000 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -275,8 +275,11 @@ class PostScriptDevice(CommonDevice):
     def SetLineAttributes(self, width, cap = 1, join = 0, dashes = ()):
 	write = self.file.write
 	if self.current_width != width:
+            width_changed = 1
 	    write('%g w\n' % width)
 	    self.current_width = width
+        else:
+            width_changed = 0
 	join = ps_join[join]
 	if self.current_join != join:
 	    write('%d j\n' % join)
@@ -286,11 +289,11 @@ class PostScriptDevice(CommonDevice):
 	    write('%d J\n' % cap)
 	    self.current_cap = cap
 
-	if self.current_dash != dashes:
+	if self.current_dash != dashes or width_changed:
 	    # for long dashes tuples this could theoretically produce
 	    # lines longer than 255 chars, which means that the file
 	    # would not conform to the DSC
-	    if width == 0:
+	    if width < 1:
 		width = 1
 	    write('[')
 	    for dash in dashes:
