@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2000 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -34,7 +34,7 @@ from Sketch.warn import pdebug, warn, INTERNAL
 from Sketch import const
 from Sketch.const import SelectSet
 
-from Sketch import _, Point, Rect, UnionRects, RectType, Identity, \
+from Sketch import _, Point, Polar, Rect, UnionRects, RectType, Identity, \
      Trafo, TrafoType, Rotation, CreateListUndo, NullUndo
 
 import handle
@@ -291,6 +291,11 @@ class Selection(Bounded):
 		last_info = info
 	return changed
 
+    def update_selinfo(self):
+        objs = self.objects
+        for i in range(len(objs)):
+            objs[i] = objs[i][-1].SelectionInfo()
+
     def SetSelection(self, info):
 	old_objs = self.objects
 	if info:
@@ -556,11 +561,10 @@ class SizeRectangle(SelectionRectangle):
 			w = sign * h / aspect
 		    p = Point(ref_x + w, ref_y + h)
 	    elif self.selection == -1:
+		pi4 = math.pi / 4
 		off = p - self.drag_start
-		if abs(off.x) > abs(off.y):
-		    p = Point(p.x, self.drag_start.y)
-		else:
-		    p = Point(self.drag_start.x, p.y)
+                d = Polar(pi4 * round(math.atan2(off.y, off.x) / pi4))
+                p = self.drag_start + (off * d) * d
 	return p
 
     def ButtonDown(self, p, button, state):
