@@ -1,5 +1,5 @@
 /* Sketch - A Python-based interactive drawing program
- * Copyright (C) 1997, 1998, 1999 by Bernhard Herzog
+ * Copyright (C) 1997, 1998, 1999, 2000 by Bernhard Herzog
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -231,7 +231,7 @@ curve_compare(SKCurveObject * v, SKCurveObject * w)
     if (v == w)
 	return 0;
 
-    return (int)v < (int)w ? -1 : +1;
+    return v < w ? -1 : +1;
 }
 
 static PyObject *
