@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1996, 1997, 1998, 1999 by Bernhard Herzog
+# Copyright (C) 1996, 1997, 1998, 1999, 2000 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -128,6 +128,7 @@ class SketchCanvas(SketchView, CursorStack, WidgetWithModes):
 		 document = None, **kw):
 	self.init_handles()
 	self.init_modes()
+        self.init_cross_hairs()
 	WidgetWithModes.__init__(self)
 	kw['show_visible'] = 1
 	kw['show_printable'] = 0
@@ -238,9 +239,13 @@ class SketchCanvas(SketchView, CursorStack, WidgetWithModes):
     def LeaveEvent(self, event):
 	if event.state & const.Button1Mask:
 	    self.start_auto_scroll()
+        self.hide_crosshairs()
+        self.crosshairs_visible = 0
 
     def EnterEvent(self, event):
 	self.cancel_auto_scroll()
+        self.crosshairs_visible = 1
+        self.show_crosshairs()
 
     def create_obj_cmds(self, commands):
 	cmds = command.Commands()
@@ -398,6 +403,7 @@ class SketchCanvas(SketchView, CursorStack, WidgetWithModes):
             self.set_handle_cursor(x, y)
 
     def doc_was_edited(self, *rest):
+        self.selection_info_text = None
 	self.update_snap_points()
 	self.update_handles()
 	self.queue_message(EDITED)
@@ -451,6 +457,7 @@ class SketchCanvas(SketchView, CursorStack, WidgetWithModes):
 	    self.current.DrawDragged(self.invgc, 0)
 	else:
 	    self.show_handles(1)
+        self.show_crosshairs(1)
 	if region:
 	    self.invgc.PopClip()
 	#self.show_handles()
@@ -495,6 +502,8 @@ class SketchCanvas(SketchView, CursorStack, WidgetWithModes):
 	else:
 	    if self.cursor_function:
 		self.cursor_function(event.x, event.y)
+        self.hide_crosshairs()
+        self.show_crosshairs()
 
     def ButtonReleaseEvent(self, event):
 	self.grab_release()
@@ -1232,13 +1241,36 @@ class SketchCanvas(SketchView, CursorStack, WidgetWithModes):
 	    obj.ButtonUp(self.correct_and_snap(p), button, state)
 	    self.current = None
 
+        zoom_out = state & const.ControlMask
+
 	self.save_viewport()
-	if self.dragging and obj is not None:
-	    self.zoom_fit_rect(obj.bounding_rect)
-	else:
-	    scale = min(2 * self.gc.scale, self.max_scale)
-	    self.SetScale(scale / self.pixel_per_point, do_center = 0)
-	    self.SetCenter(p, move_contents = 0)
+        if self.dragging and obj is not None:
+            epsilon = 1e-10
+            rect = obj.bounding_rect
+            rw = rect.right - rect.left
+            rh = rect.top - rect.bottom
+            width = self.tkwin.width
+            height = self.tkwin.height
+            if abs(rw) < epsilon or abs(rh) < epsilon:
+                return
+            scalex = width / rw
+            scaley = height / rh
+            if zoom_out:
+                scale = (self.scale ** 2) / max(scalex, scaley)
+                x, y = self.DocToWin(rect.center())
+                center = rect.center() + Point(width / 2 - x,
+                                               -height / 2 + y) / scale
+            else:
+                scale = min(scalex, scaley, self.max_scale)
+                center = rect.center()
+        else:
+            if zoom_out:
+                scale = 0.5 * self.scale
+            else:
+                scale = 2 * self.scale
+            center = p
+        self.SetScale(scale / self.pixel_per_point, do_center = 0)
+        self.SetCenter(center, move_contents = 0)
 
 	self.start_drag = 0
 	self.dragging = 0
@@ -1247,8 +1279,8 @@ class SketchCanvas(SketchView, CursorStack, WidgetWithModes):
     #	Other mode related methods
     #
 
-    AddCmd('ToggleMode', _("Toggle Mode"), key_stroke = ' ',
-	   subscribe_to = None, is_check = 1)
+    AddCmd('ToggleMode', _("Toggle Mode"), key_stroke = (' ', 'C-space'),
+	   subscribe_to = None)
     def ToggleMode(self):
 	self.begin_transaction()
 	try:
@@ -1365,7 +1397,13 @@ class SketchCanvas(SketchView, CursorStack, WidgetWithModes):
 	    pgrid = self.document.SnapToGrid(p)
 	if self.snap_to_guide:
 	    pgh, pgv, pguide = self.document.SnapToGuide(p, maxdist)
-	    pguide = min(pgh, pgv, pguide)
+            if (self.current is None
+                or isinstance(self.current, SelectionRectangle)
+                or not self.current.is_GuideLine):
+                # We're not dragging a guideline, so snap to guide-lines
+                # or to guide-objects. Otherwise, only snap to guide
+                # objects
+                pguide = min(pgh, pgv, pguide)
 	    if pguide == pgh or pguide == pgv:
 		dist, (x, y) = pguide
 		if x is None:
@@ -1490,6 +1528,52 @@ class SketchCanvas(SketchView, CursorStack, WidgetWithModes):
 	    return _("Snap: ") + string.join(snap, '/')
 	return _("No Snap")
 
+    #
+    #   Crosshairs
+    #
+
+    def init_cross_hairs(self):
+        self.crosshairs = 0
+        self.crosshairs_visible = 0
+        self.crosshairs_pos = ()
+        self.crosshairs_drawn = 0
+
+    def draw_crosshairs(self, pos):
+        x, y = self.DocToWin(pos)
+        width = self.tkwin.width
+        height = self.tkwin.height
+        self.invgc.gc.DrawLine(0, y, width, y)
+        self.invgc.gc.DrawLine(x, 0, x, height)
+        self.crosshairs_pos = pos
+
+    def show_crosshairs(self, force = 0):
+        if self.crosshairs and self.crosshairs_visible:
+            if not self.crosshairs_drawn or force:
+                self.draw_crosshairs(self.current_pos)
+            self.crosshairs_drawn = 1
+
+    def hide_crosshairs(self):
+        if self.crosshairs_drawn:
+            self.draw_crosshairs(self.crosshairs_pos)
+        self.crosshairs_drawn = 0
+
+    AddCmd('ToggleCrosshairs', _("Crosshairs"), key_stroke = 'F2',
+	   value = 0, value_cb = 'IsShowingCrosshairs', is_check = 1)
+    def ToggleCrosshairs(self):
+	self.begin_transaction()
+	try:
+            self.hide_crosshairs()
+            self.crosshairs = not self.crosshairs
+            self.show_crosshairs()
+            self.issue_state()
+	finally:
+	    self.end_transaction()
+
+    def IsShowingCrosshairs(self):
+	return self.crosshairs
+
+
+    
     #
     #	Handles
     #
@@ -1531,14 +1615,14 @@ class SketchCanvas(SketchView, CursorStack, WidgetWithModes):
 		p = p + Point(offset[0] * factor, offset[1] * factor)
 
 	    if handle_type == HandleLine:
-		funcs.append(self.invgc.DrawHandleLine, (p, handle.p2))
+		funcs.append((self.invgc.DrawHandleLine, (p, handle.p2)))
 	    elif handle_type == Handle_Pixmap:
 		if cursor:
 		    points.append(self.DocToWin(p) + (cursor, handle))
-		funcs.append(self.draw_handle_funcs[handle_type],
-			     (p, handle.pixmap))
+		funcs.append((self.draw_handle_funcs[handle_type],
+                              (p, handle.pixmap)))
 	    elif handle_type == Handle_Caret:
-		funcs.append(self.invgc.DrawCaretHandle, (p, handle.p2))
+		funcs.append((self.invgc.DrawCaretHandle, (p, handle.p2)))
 	    elif handle_type == Handle_SmallOpenRectList:
 		p = handle.list
 		if p:
